@@ -20,11 +20,11 @@
 #' set.seed(1)
 #' X <- rsparsematrix(5, 3, .5, repr="R")
 #' X[1:3] <- 0
-#' X
+#' print(X)
 NULL
 
 throw_shape_err <- function() {
-    stop("Values to assign does not match with matrix dimensions.")
+    stop("Values to assign do not match with matrix dimensions.")
 }
 
 assign_through_matrix <- function(x, i, j, value) {
@@ -49,8 +49,6 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
     if (is.matrix(value))
         value <- as.numeric(value)
 
-    tot_x <- nrow(x) * ncol(x)
-
 
     if (is.null(ij_properties))
         ij_properties <- get_ij_properties(x, i ,j)
@@ -78,15 +76,33 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
             return(assign_through_matrix(x, i, j, value))
     }
 
+    if (
+        inherits(value, c("sparseVector", "sparseMatrix")) &&
+        ((.hasSlot(value, "x") && length(value@x) == 0) ||
+         (!.hasSlot(value, "x") && .hasSlot(value, "i") && length(value@i) == 0) ||
+         (!.hasSlot(value, "x") && !.hasSlot(value, "i") && length(value@j) == 0))
+    ) {
+        if (inherits(value, "sparseMatrix")) {
+            if (!check_shapes_are_assignable_2d(length(i), length(j), nrow(value), ncol(value)))
+                throw_shape_err()
+        } else if (inherits(value, "sparseVector")) {
+            if (!check_shapes_are_assignable_1d(length(i), length(j), value))
+                throw_shape_err()
+        } else {
+            throw_internal_error()
+        }
+        value <- 0.
+    }
+
     if (inherits(value, c("numeric", "integer", "logical")) &&
         length(value) > 1 && length(i) > 1 && length(j) > 1 &&
-        length(value) == length(i) * length(j)
+        (length(value) / length(i)) == length(j)
     ) {
         value <- matrix(as.numeric(value), nrow=length(i), ncol=length(j))
     }
     if (inherits(value, "sparseVector") &&
         length(value) > 1 && length(i) > 1 && length(j) > 1 &&
-        length(value) == (length(i) * length(j))
+        (length(value) / length(i)) == length(j)
     ) {
         value <- Matrix(value, nrow=length(i), ncol=length(j), byrow=FALSE)
     }
@@ -109,7 +125,8 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
                 X_attr$x <- numeric()
                 X_attr$j <- integer()
                 X_attr$p <- integer(X_attr$Dim[1L]+1L)
-                return(X_attr)
+                attributes(x) <- X_attr
+                return(x)
             }
 
             else if (all_j) {
@@ -234,7 +251,7 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
         value <- as.numeric(value)
 
         if (all_i && all_j) {
-            if ((length(value) > tot_x) || (tot_x %% length(value)) != 0)
+            if (!check_shapes_are_assignable_1d(nrow(x), ncol(x), value))
                 throw_shape_err()
             warning("Warning: attempting to set all coordinates of a sparse matrix.")
             res <- matrix(value, nrow=nrow(x), ncol=ncol(x))
@@ -274,7 +291,7 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
         }
 
         else {
-            if (length(i) * length(j) != length(value))
+            if (!check_shapes_are_assignable_1d(length(i), length(j), value))
                 throw_shape_err()
 
             return(assign_through_matrix(x, i, j, value))
@@ -289,7 +306,7 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
             return(assign_csr_internal(x, i, j, as.numeric(value), ij_properties))
 
         if (all_i && all_j) {
-            if ((length(value) > tot_x) || (tot_x %% length(value)) != 0)
+            if (!check_shapes_are_assignable_1d(nrow(x), ncol(x), value))
                 throw_shape_err()
             return(assign_through_matrix(x, i, j, value))
         }
@@ -335,12 +352,11 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
 
     else if (inherits(value, "sparseMatrix"))
     {
-        tot_val <- nrow(value) * ncol(value)
         if (length(value@x) == 0L)
             return(assign_csr_internal(x, i, j, 0, ij_properties))
         
         if (all_i && all_j) {
-            if (tot_val > tot_x || (tot_x %% tot_val) != 0)
+            if (!check_shapes_are_assignable_2d(nrow(x), ncol(x), nrow(value), ncol(value)))
                 throw_shape_err()
             if (nrow(x) == nrow(value) && ncol(x) == ncol(value))
                 return(as.csr.matrix(value))
@@ -350,7 +366,7 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
 
         else if (all_j) {
             if (length(i) == 1L) {
-                if (tot_val > ncol(x) || (ncol(x) %% tot_val) != 0)
+                if (!check_shapes_are_assignable_1d_v2(ncol(x), nrow(value), ncol(value)))
                     throw_shape_err()
                 if (nrow(value) != 1L && ncol(value) != 1L) {
                     value <- as.sparse.vector(value)
@@ -425,7 +441,7 @@ assign_csr_internal <- function(x, i, j, value, ij_properties=NULL) {
 
         else if (all_i) {
             if (length(j) == 1L) {
-                if (tot_val > nrow(x) || (nrow(x) %% tot_val) != 0)
+                if (!check_shapes_are_assignable_1d_v2(nrow(x), nrow(value), ncol(value)))
                     throw_shape_err()
                 if (nrow(value) != 1L && ncol(value) != 1L) {
                     value <- as.sparse.vector(value)
